<?php
/**
 * WooCommerce Extension Boilerplate order functions and filters.
 *
 * @class 	WC_Affilae_Order
 * @version 1.0.0
 * @since   1.0.0
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class WC_Affilae_Order {

	/**
	 * Setup order class
	 */
	public function __construct() {
		add_action( 'woocommerce_thankyou', array( $this, 'affilae_conversion_tracking' ), 10, 3 );
	}


	/**
	 * Fetch affilae rules to display
	 *
	 * @access public
	 *
	 * @param  int $order_id
	 *
	 * @return void
	 */
	public function affilae_conversion_tracking( $order_id ) {
		$order = new WC_Order($order_id);

		$order_total = (float) $order->get_total() - $order->get_total_tax() - $order->get_total_shipping();

		$rules    = $this->getRules($order);
		$payment  = $this->getPayment($order);
		$order_id = $order->get_order_number();

		$key = get_option( 'wc_affilae_program_id' );
		$voucher = $this->getOrderVouchers($order);

		// display the template
		wc_get_template(
			'order/tracking.php',
			array(
				'key' => $key,
				'payment' => $payment,
				'rules' => $rules,
				'order_id' => $order_id,
				'voucher' => $voucher,
			),
			FALSE,
			WC_Affilae::$dir . '/templates/' );
	}

	/**
	 * @param $order WC_Order
	 * @return string
	 */
	public function getOrderVouchers($order)
	{
		$vouchers = "";
		$vouchersArray = array();
		foreach ($order->get_used_coupons() as $coupon_name) {
			$vouchersArray[] = $coupon_name;
		}

		if (count($vouchersArray) > 0) {
			$vouchers = implode(';', $vouchersArray);
		}

		return $vouchers;
	}

	public function getOrderCategories($order) {
		$categories = array();
		$items = $order->get_items();

		foreach ($items as $item) {
			$product_id = $item['product_id'];
			$terms = get_the_terms( $product_id, 'product_cat' );

			if (is_array($terms) && count($terms) > 0) {
				foreach ($terms as $term) {
					$categories[] = $term->term_id;
				}
			}
		}

		return $categories;
	}


	/**
	 * Get tracking rules
	 *
	 * @access public
	 *
	 * @param  WC_Order $order
	 *
	 * @return array
	 */
	public function getRules($order) {
		$order_total = (float) $order->get_total() - $order->get_total_tax() - $order->get_total_shipping();
		$order_total_remaining = $order_total;

		$rules = array();
		$category_rules = array();
		$generic_rules = array();

		$args = array(
			'post_status' => array( 'publish' ),
			'post_type'   => 'affilae_rule'
		);

		$query = new WP_Query( $args );

		while ( $query->have_posts() ) : $query->the_post();
			$rule_id         = get_the_id();
			$rule_categories = get_the_terms( $rule_id, 'product_cat' );

			if ( is_array($rule_categories) && count($rule_categories) > 0 ) {
				// Only add the first category
				foreach ( $rule_categories as $rule_category ) {
					$category_rules[$rule_category->term_id] = get_post();
					break;
				}
			} elseif ( $rule_categories === false ) {
				$generic_rules[] = get_post();
			}
		endwhile;


		if ( count($category_rules) > 0 ) {
			$order_categories = $this->getOrderCategories( $order );
			$category_total = array();

			$items = $order->get_items();

			if (is_array($items) && count($items) > 0) {
				foreach ($items as $item) {
					if (version_compare(WC_VERSION, 3.0, '<')) {
						$product_id = $item['product_id'];
					} else {
						$product_id = $item->get_product_id();
					}

					$terms = get_the_terms( $product_id, 'product_cat' );

					foreach ($terms as $term) {
						$category_id = $term->term_id;
						break;
					}

					if ( array_key_exists($category_id, $category_rules) ) {
						$rule = $category_rules[$category_id];

						if (version_compare(WC_VERSION, 3.0, '<')) {
							$line_total = round($item['item_meta']['_line_total'][0], 2);
						} else {
							$line_total = round($item->get_total(), 2);
						}

						$order_total_remaining -= (float) $line_total;

						$rules[] = array(
							'code'   => get_post_meta( $rule->ID, '_code', true ),
							'amount' => $line_total
						);
					}
				}
			}

			if ( count($generic_rules) > 0 && $order_total_remaining > 0) {
				$rule = $generic_rules[0];

				$rules[] = array(
					'code'   => get_post_meta( $rule->ID, '_code', true ),
					'amount' => $order_total_remaining
				);
			}
		} elseif ( count($generic_rules) > 0 ) {
			foreach ( $generic_rules as $rule ) {
				$rules[] = array(
					'code'   => get_post_meta( $rule->ID, '_code', true ),
					'amount' => $order_total
				);
			}
		}

		return $rules;
	}


	/**
	 * Get the payment method name
	 *
	 * @access public
	 *
	 * @param  WC_Order $order_id
	 *
	 * @return void
	 */
	public function getPayment( $order ) {
		$payment = get_post_meta( $order->get_order_number(), '_payment_method', true );

		switch ($payment) {
			case 'bacs':
				$payment = 'bankwire';
				break;

			case 'allopass':
			case 'atos':
			case 'buyster':
			case 'cetrel':
			case 'clickandbuy':
			case 'cmcic':
			case 'etransactions_std':
			case 'hipay':
			case 'kwixo':
			case 'paybox':
			case 'paygate':
			case 'payline':
			case 'paypal':
			case 'paypoint':
			case 'receiveandpay':
			case 'rentabiliweb':
			case 'securepay':
			case 'sips':
			case 'sofinco':
			case 'spplus':
			case 'starpass':
			case 'systempay':
			case 'vads':
				$payment = 'online';
				break;

			default:
				$payment = 'other';
				break;
		}

		return $payment;
	}

}
